package name.dhiller.jBorderLiner;

import com.sun.star.awt.XWindow;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.table.XCell;
import com.sun.star.sheet.XCellAddressable;
import com.sun.star.accessibility.*;
//import javax.swing.JOptionPane;             // for Messagebox TEST only

/** Borders handling
 * 
 * @author Dietmar Hiller (Dhiller)
 * @author Max Giesbert (co-author)
 * @version 080709
 */
public final class BLMouseClickHandlerImpl extends WeakBase
   implements com.sun.star.awt.XEnhancedMouseClickHandler
{
    private final XComponentContext m_xContext;
    private final com.sun.star.frame.XFrame m_xFrame;
    private static final String m_implementationName = BLMouseClickHandlerImpl.class.getName();
    private final short LIMIT=6;  // limit for border identification

    public BLMouseClickHandlerImpl(com.sun.star.frame.XFrame xFrame,
            XComponentContext xContext)
    {
        m_xFrame = xFrame;
        m_xContext = xContext;
    };

    // com.sun.star.lang.XEventListener:
    public void disposing(com.sun.star.lang.EventObject Source)
    {
        // TODO: Insert your implementation for "disposing" here.
    }

    // com.sun.star.awt.XEnhancedMouseClickHandler:
    public boolean mousePressed(com.sun.star.awt.EnhancedMouseEvent e)
    {
        // TODO: Exchange the default return implementation for "mousePressed" !!!
        // NOTE: Default initialized polymorphic structs can cause problems
        // because of missing default initialization of primitive types of
        // some C++ compilers or different Any initialization in Java and C++
        // polymorphic structs.
        return false;   // forward mouse event
    }

    public boolean mouseReleased(com.sun.star.awt.EnhancedMouseEvent e)
    {
        // TODO: Exchange the default return implementation for "mouseReleased" !!!
        // NOTE: Default initialized polymorphic structs can cause problems
        // because of missing default initialization of primitive types of
        // some C++ compilers or different Any initialization in Java and C++
        // polymorphic structs.
        short Modifiers=0;
        if (e.ClickCount > 1) return false; // ignore multiple clicks;
        switch (e.Buttons) {
            case com.sun.star.awt.MouseButton.LEFT:
               Modifiers = com.sun.star.awt.KeyModifier.MOD2;   //ALT
               break;
            /* right click always opens also context menu, cannot make it stop
            case com.sun.star.awt.MouseButton.RIGHT:
               Modifiers = com.sun.star.awt.KeyModifier.SHIFT;
               break;
            case com.sun.star.awt.MouseButton.MIDDLE:
               Modifiers = com.sun.star.awt.KeyModifier.MOD1;   //Ctrl
               break;
             */
        }                                                      
        if (Modifiers == 0) return false;   // forward mouse event

        XCell xCell = (XCell) UnoRuntime.queryInterface(XCell.class, e.Target);
        
        XWindow window = m_xFrame.getComponentWindow();
        XAccessible accessible = (XAccessible) UnoRuntime.queryInterface(XAccessible.class, window);
        XAccessibleContext accessibleContext = accessible.getAccessibleContext();
        
       	XAccessibleContext table = getAccessibleForRole(accessibleContext, 
                com.sun.star.accessibility.AccessibleRole.TABLE);
	XAccessibleTable accessibleTable = (XAccessibleTable) UnoRuntime.queryInterface(XAccessibleTable.class, table);
  
        XCellAddressable cellAddressable = (XCellAddressable) UnoRuntime.queryInterface(XCellAddressable.class, xCell);
        com.sun.star.table.CellAddress cellAddress = cellAddressable.getCellAddress();
        XAccessibleContext cellContext = null;
        try {
            cellContext = accessibleTable.getAccessibleCellAt(cellAddress.Row, cellAddress.Column).getAccessibleContext();
        } catch (com.sun.star.lang.IndexOutOfBoundsException ex) {
            ex.printStackTrace();
            return false;
        }
        XAccessibleComponent accessibleComponent = (XAccessibleComponent) UnoRuntime.queryInterface
                (XAccessibleComponent.class, cellContext);
        
	//calculate the clicked point in the cell
        int x = e.X - accessibleComponent.getLocation().X;
	int y = e.Y - accessibleComponent.getLocation().Y;
        
        //calculate the chosen border
	int width = accessibleComponent.getBounds().Width;
	int height = accessibleComponent.getBounds().Height;

        short keyCode = 0;
        if (x < LIMIT) keyCode = com.sun.star.awt.Key.NUM4; // left
        else if (x > (width - LIMIT))  keyCode = com.sun.star.awt.Key.NUM6; // right
        else if (y > (height - LIMIT)) keyCode = keyCode = com.sun.star.awt.Key.NUM2; // bottom
        else if (y < LIMIT) keyCode = com.sun.star.awt.Key.NUM8; // top
 
        if (keyCode > 0 ) Borders.DoBorders(m_xFrame, Modifiers, keyCode);  // uses selection
        //Borders.SetBordersCell(xCell, Modifiers, com.sun.star.awt.Key.NUM2); // uses Targetcell
        return true;   // forward mouse event
    }

	/**traverses the accessibility tree and returns the first {@link XAccessibleContext} with the given role
	 * @param accessibleContext the context to start searching with
	 * @param role the role of the context that should be returned
	 * @return the first XAccessibleContext found with the given role or <code>null</code>
	 * */
	private XAccessibleContext getAccessibleForRole(XAccessibleContext accessibleContext,
			short role)
	{
		XAccessibleContext ret = null;
		XAccessibleContext accessibleContext2 = null;
		int count = accessibleContext.getAccessibleChildCount();
		for (int i = 0; i < count; i++)
		{
			if(ret != null)
				return ret;
                        try {
                            accessibleContext2 = accessibleContext.getAccessibleChild(i).getAccessibleContext();
                        } catch (com.sun.star.lang.IndexOutOfBoundsException ex) {
                            ex.printStackTrace();
                            return ret;
			}
			if(accessibleContext2.getAccessibleRole() == role)
			{
				ret = accessibleContext2;
				return ret;
			}
			else
				ret = getAccessibleForRole(accessibleContext2, role);
		}
		return ret;
	}    
    
}
