/*
 * BL_TB_ActiveBtn.java
 * Created on 2. Dezember 2007, 18:44
 * BorderLiner Toolbar Active Button
 */

package name.dhiller.jBorderLiner;

import com.sun.star.frame.FeatureStateEvent;
import com.sun.star.frame.XController;
import com.sun.star.beans.NamedValue;
import com.sun.star.uno.XComponentContext;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.awt.XUserInputInterception;
                            // Mouse-Handling
import com.sun.star.sheet.XSpreadsheetView;
import com.sun.star.sheet.XEnhancedMouseClickBroadcaster;

/**
 * @author Dietmar Hiller (Dhiller)
 * @version 080419
 */
class BL_TB_ActiveBtn extends BL_ToolbarItem {
   /**
    * Constructs the button to activate Keylistening
    */
    BL_TB_ActiveBtn() {
        super();
    }
    
    /**
     * @param bMouseHandling true/false to add MouseHandler
     */
    void Init(com.sun.star.frame.XFrame xFrame,
            com.sun.star.frame.XStatusListener xControl,
            com.sun.star.util.URL aURL, XComponentContext xContext,
            boolean bMouseHandling) {

        boolean active = false;                  // state is initially not activated

        BLKeyHandlerImpl myKeyHandler = new BLKeyHandlerImpl (xFrame, xContext);
                                                // Mouse-handlilng
        XEnhancedMouseClickBroadcaster bc = null;
        BLMouseClickHandlerImpl mch = null;
        XSpreadsheetView xSpreadsheetView = (XSpreadsheetView) UnoRuntime.queryInterface(XSpreadsheetView.class, 
                    xFrame.getController());;
        if (xSpreadsheetView != null) {
           bc = (XEnhancedMouseClickBroadcaster) UnoRuntime.queryInterface(XEnhancedMouseClickBroadcaster.class, 
                    xSpreadsheetView);  
           if ((bc != null) && bMouseHandling) {
                mch = new BLMouseClickHandlerImpl(xFrame, xContext);
           }
        }
        BLHandlers myBLHandlers = new BLHandlers(myKeyHandler, bc, mch);
        
        super.Init (xFrame, xControl, aURL, myBLHandlers);

        FeatureStateEvent aState = new FeatureStateEvent();
        aState.FeatureURL = aURL;
        aState.IsEnabled  = true;
        aState.State      = active;
        xControl.statusChanged(aState);
    }
        
    /** dispatch - react on click on Active-Button
     *      active contains state 
     * @param aURL this URL
     * @param aArguments .Name = "KeyModifier", 
     *                   .Name = "ToggleButton", .Value = true/false
     * @param xFrame - this Frame
     */ 
    void dispatch( 
            com.sun.star.util.URL aURL,
            com.sun.star.beans.PropertyValue[] aArguments,
            com.sun.star.frame.XFrame xFrame)
    { 
                                                            // toggle the button
        boolean active = getControlElementFromFrame(xFrame).getState();
        active  = !active;           
        setStatus(xFrame, active);
    }
    
    /**
     * set status of button and (de-)activate KeyHandler
     * @param xFrame - frame the button is in
     * @param status    - true: button pressed, Keyhandler active
     *                  - false: button released, Keyhandler removed
     */
    void setStatus(com.sun.star.frame.XFrame xFrame, boolean status) {
        FeatureStateEvent aState = new FeatureStateEvent();
        aState.FeatureURL = m_URL;
        boolean active  = status;              
        aState.IsEnabled  = true;
        aState.State      = active;
        changeStatusFrame(xFrame, aState);
        
        ContextElement ce = getControlElementFromFrame(xFrame);
        ce.setState(active);

        //###BLKeyHandlerImpl myKeyHandler = (BLKeyHandlerImpl)getControlElementFromFrame(xFrame).getObject();
        BLHandlers myBLHandlers = (BLHandlers)ce.getObject();
        BLKeyHandlerImpl myKeyHandler = myBLHandlers.getKeyHandler();   // get key handler
        XEnhancedMouseClickBroadcaster myeMCBc = myBLHandlers.getEMCBc();   // get mouseclickBroadcaster
        
        if (active) {
            //http://lxr.go-oo.org/source/framework/framework/qa/complex/XUserInputInterception/EventTest.java   
            XController xController = xFrame.getController();
            XUserInputInterception xUII = (XUserInputInterception) UnoRuntime.queryInterface(
                 XUserInputInterception.class, xController);
            xUII.addKeyHandler(myKeyHandler);    
            if (myeMCBc != null) {
                BLMouseClickHandlerImpl mch = myBLHandlers.getEMCH();
                // BLMouseClickHandlerImpl mch = new BLMouseClickHandlerImpl();
                if (mch != null) myeMCBc.addEnhancedMouseClickHandler(mch);
                // myBLHandlers.setEMCH(mch);
                // ce.setObject(myBLHandlers);
            }
        }
        else {
            XController xController = xFrame.getController();
            XUserInputInterception xUII = (XUserInputInterception) UnoRuntime.queryInterface(
                 XUserInputInterception.class, xController);
            xUII.removeKeyHandler(myKeyHandler); 
            if(myeMCBc != null) { 
                BLMouseClickHandlerImpl mch = myBLHandlers.getEMCH();
                if (mch != null) {
                    myeMCBc.removeEnhancedMouseClickHandler(mch);
                    // mch = null;
                    // myBLHandlers.setEMCH(mch);
                    // ce.setObject(myBLHandlers);
                }
            }
                
        }
    }
}
    
/******************************************************************************* 
 * class to hold the keyhandler, mousebroadcaster and mousehandler
 */
class BLHandlers {
    BLKeyHandlerImpl BLKeyHandler;
    XEnhancedMouseClickBroadcaster BLeMCBc;
    BLMouseClickHandlerImpl BLMCH;
    
    /** Constructor
     */
    BLHandlers (BLKeyHandlerImpl aBLKeyHandler, XEnhancedMouseClickBroadcaster aBLeMCBc, 
            BLMouseClickHandlerImpl aBLMCH)
    {
        BLKeyHandler = aBLKeyHandler;
        BLeMCBc = aBLeMCBc;
        BLMCH = aBLMCH;
    }
    
    /** get KeyHandler
     */
    BLKeyHandlerImpl getKeyHandler() {
        return BLKeyHandler;
    }       

    /** get EnhancedMouseClickBroadcaster
     */
    XEnhancedMouseClickBroadcaster getEMCBc() {
        return BLeMCBc;
    }  

    /** get MouseClickHandler
     */
    BLMouseClickHandlerImpl getEMCH() {
        return BLMCH;
    }      

    /** set MouseClickHandler
     */
    void setEMCH(BLMouseClickHandlerImpl aBLMCH) {
        BLMCH = aBLMCH;
        return;
    }      
    
    /**com.sun.star.frame.XStatusListener m_xControl;
    com.sun.star.frame.XFrame m_xFrame;
    Object m_oObject;
    boolean m_state;

    /** Constructor, state is by default false
     * @param aFrame Frame of Control
     * @param xControl Control
     * @param oObject any Object associated to that control
     *
    ContextElement( com.sun.star.frame.XFrame aFrame, 
            com.sun.star.frame.XStatusListener xControl, Object oObject)
    {
        m_xControl = xControl;
        m_xFrame = aFrame;
        m_oObject = oObject;
        m_state = false;
    }

    /** get the context
     * @deprecated use getControl() instead
     
    @Deprecated
    com.sun.star.frame.XStatusListener getContext() {
        return m_xControl;
    }
    
    /** get the Frame
     *
    com.sun.star.frame.XFrame getFrame() {
        return m_xFrame;
    }

    /** get the Object
     * @return Object
     *
    Object getObject() {
        return m_oObject;
    }
    
    /** set the Object
     * @param oObject
     *
    void setObject(Object oObject) {
        m_oObject = oObject;
    }
    
    /** get state
     * @return state
     *
    boolean getState() {
        return m_state;
    }
    
    /** set state
     * @param state e.g. of a pressed toolbarbutton
     *
    void setState(boolean state) {
        m_state = state;
    }
    /** toString
    public String toString()
    {
        return m_xControl.toString();
    }*/    
}