/*
 * BL_ToolbarItem.java
 *
 * Created on 2. Dezember 2007, 18:10
 *
 * BorderLiner Toolbar Items
 */

package name.dhiller.jBorderLiner;

import com.sun.star.beans.NamedValue;
import com.sun.star.frame.ControlCommand;
import com.sun.star.frame.FeatureStateEvent;
import com.sun.star.frame.XStatusListener;
import java.util.Vector;

/**
 * ToolbarItem
 *  this concept could cause problems in a multi-thread / multi-frame environment
 *  see Developer's Guide, p. 277 (4.7.1. Protocol Handler)
 * @author dhiller
 * @version 080117
 * @since OOo 2.0.3 (controlCommand)
 */
abstract class BL_ToolbarItem {
    com.sun.star.util.URL m_URL;
    private Vector contextAssignment;
    private boolean bEnabled;
    
    /** Creates a new instance of BL_ToolbarItem */
    BL_ToolbarItem() {
        contextAssignment = new Vector();
    }
    
    /** Initialize a toolbar item
     * @param aFrame Frame of Control
     * @param xControl - unique identifier of toolbar per document
     * @param aURL - URL of that feature (unique identifier of item within toolbar, not per document)
     * @param oObject - Object linked to that toolbar item
     */
    void Init(com.sun.star.frame.XFrame xFrame,
              com.sun.star.frame.XStatusListener xControl,
              com.sun.star.util.URL aURL, Object oObject) {
        m_URL = aURL;
        contextAssignment.addElement(new ContextElement(xFrame, xControl, oObject));
        bEnabled = true;
    }
    
    /**
     */
    void dispatch( com.sun.star.util.URL aURL,
                   com.sun.star.beans.PropertyValue[] aArguments ) {
        ;
    }
        
    /** send a command to all xControl(s)
     * @param command - e.g. "SetValues"
     * @param rArgs - e.g. "Value" - icurrentLineStyle
     */
    final void sendCommandToAll(String command, NamedValue[] rArgs) {
        com.sun.star.frame.XStatusListener xControl;
        ContextElement ce;
        for (int i=0; i<contextAssignment.size(); i++) {
            ce = (ContextElement)contextAssignment.elementAt(i);
            xControl = ce.getControl();
            SendCommandTo(xControl, command, rArgs);
        }
    }

    /**
     * send a command to a particular control
     * @param xControl Control to send command to
     * @param rCommand Command
     * @param rArgs command specific arguments
     */
    final void SendCommandTo(com.sun.star.frame.XStatusListener xControl,
                    String rCommand, NamedValue[] rArgs)
    {
        FeatureStateEvent aEvent = new FeatureStateEvent();
        aEvent.FeatureURL = m_URL; 
        // aEvent.Source = (::com::sun::star::frame::XDispatch*) this;
        aEvent.IsEnabled  = bEnabled;
        aEvent.Requery    = false;
        ControlCommand aCtrlCmd = new ControlCommand();
        aCtrlCmd.Command   = rCommand;
        aCtrlCmd.Arguments = rArgs;
        aEvent.State = aCtrlCmd;
        xControl.statusChanged( aEvent );
    }

    /** remove context (e.g. window was closed)
     * @param xControl - control to be removed
     * @param aURL - URL, not used
     */
    final void removeContext( com.sun.star.frame.XStatusListener xControl, 
                  com.sun.star.util.URL aURL) {
        int i = hasContext(xControl);
        if (i >= 0) {
            ContextElement ce = (ContextElement)contextAssignment.elementAt(i);
            ce.m_oObject = null;        // remove reference to Object first
            contextAssignment.removeElementAt(i);
        }
    }

    /** checks, if context already exists.
     *  This is necessary, since equals of the Elements is not overwritten
     * @param xControl
     * @return index where xControl is in the list
     */
    private final int hasContext(com.sun.star.frame.XStatusListener xControl) {
        int found = -1;
        for (int i=0; i<contextAssignment.size(); i++) {
            ContextElement ce = (ContextElement)contextAssignment.elementAt(i);
            com.sun.star.frame.XStatusListener listControl = ce.getControl();
            if ((listControl == xControl) ) {
                found = i;
                break;
            }
        }
        return found;
    }

    /** checks, if context already exists.
     *  This is necessary, since equals of the Elements is not overwritten
     * @param xControl
     * @return index where xControl is in the list
     */
    final ContextElement getControlElement(com.sun.star.frame.XStatusListener xControl) {
        ContextElement ce=null;
        for (int i=0; i<contextAssignment.size(); i++) {
            ce = (ContextElement)contextAssignment.elementAt(i);
            com.sun.star.frame.XStatusListener listControl = ce.getControl();
            if ((listControl == xControl) ) {
                return ce;
            }
        }
        return null;
    }

    /**
     * get the frame of a control
     * @param xControl Control to get the frame for
     * @return XFrame of that control
     */
    final com.sun.star.frame.XFrame GetFrame(com.sun.star.frame.XStatusListener xControl) {
        com.sun.star.frame.XFrame xFrame = null;
        int i = hasContext(xControl);
        if (i >= 0) {
            ContextElement ce = (ContextElement)contextAssignment.elementAt(i);
            xFrame = ce.m_xFrame;
        }
        return xFrame;
    }
     
    /** change the status of a toolbaritem of a particular Frame
     * @param xFrame Frame to send command to
     * @param aState new FeatureState
     */
    final void changeStatusFrame(com.sun.star.frame.XFrame xFrame,
            FeatureStateEvent aState) {
        com.sun.star.frame.XStatusListener xControl;
        ContextElement ce;
        for (int i=0; i<contextAssignment.size(); i++) {
            ce = (ContextElement)contextAssignment.elementAt(i);
            if (ce.m_xFrame == xFrame) {
                xControl = ce.getControl();
                xControl.statusChanged(aState);
            }
        }
    }

    /**
     * get the control of a frame
     * @param XFrame Frame to get the control for
     * @return xControl control of that frame
     */
    final com.sun.star.frame.XStatusListener GetControl(com.sun.star.frame.XFrame xFrame) {
        com.sun.star.frame.XStatusListener xControl = null;
        ContextElement ce=getControlElementFromFrame (xFrame);
        if (ce != null) xControl = ce.getControl();
        return xControl;
    }

    /** gets the controlElement from the frame
     * @param XFrame
     * @return ControlElement or null if frame does not exist
     */
    final ContextElement getControlElementFromFrame(com.sun.star.frame.XFrame xFrame) {
        ContextElement ce=null;
        for (int i=0; i<contextAssignment.size(); i++) {
            ce = (ContextElement)contextAssignment.elementAt(i);
            com.sun.star.frame.XFrame listFrame = ce.getFrame();
            if ((listFrame == xFrame) ) {
                return ce;
            }
        }
        return null;
    }
}

/** vector element to collect the xControl(s)
 *  NB: indexOf cannot be used because equals is not overwritten
 */
class ContextElement {
    com.sun.star.frame.XStatusListener m_xControl;
    com.sun.star.frame.XFrame m_xFrame;
    Object m_oObject;
    boolean m_state;

    /** Constructor, state is by default false
     * @param aFrame Frame of Control
     * @param xControl Control
     * @param oObject any Object associated to that control
     */
    ContextElement( com.sun.star.frame.XFrame aFrame, 
            com.sun.star.frame.XStatusListener xControl, Object oObject)
    {
        m_xControl = xControl;
        m_xFrame = aFrame;
        m_oObject = oObject;
        m_state = false;
    }

    /** get the context
     * @deprecated use getControl() instead
     */
    @Deprecated
    com.sun.star.frame.XStatusListener getContext() {
        return m_xControl;
    }

    /** get the Control
     */
    com.sun.star.frame.XStatusListener getControl() {
        return m_xControl;
    }
    
    /** get the Frame
     */
    com.sun.star.frame.XFrame getFrame() {
        return m_xFrame;
    }

    /** get the Object
     * @return Object
     */
    Object getObject() {
        return m_oObject;
    }
    
    /** set the Object
     * @param oObject
     */
    void setObject(Object oObject) {
        m_oObject = oObject;
    }
    
    /** get state
     * @return state
     */
    boolean getState() {
        return m_state;
    }
    
    /** set state
     * @param state e.g. of a pressed toolbarbutton
     */
    void setState(boolean state) {
        m_state = state;
    }
    /** toString
    public String toString()
    {
        return m_xControl.toString();
    }*/
}